@@ -6,6 +6,7 @@
 //! [`julia.h`]: https://github.com/JuliaLang/julia/blob/96786e22ccabfdafd073122abb1fb69cea921e17/src/julia.h#L380
 
 use super::{private::Wrapper, SymbolRef};
+use crate::wrappers::ptr::symbol::Symbol;
 use crate::wrappers::ptr::{MethodTableRef, ModuleRef, SimpleVectorRef, ValueRef};
 use crate::{impl_debug, impl_julia_typecheck, impl_valid_layout};
 use crate::{memory::global::Global, private::Private};
@@ -13,8 +14,18 @@ use jl_sys::{
     jl_array_typename, jl_llvmpointer_typename, jl_namedtuple_typename, jl_pointer_typename,
     jl_tuple_typename, jl_type_typename, jl_typename_t, jl_typename_type, jl_vecelement_typename,
 };
+
+#[cfg(not(feature = "lts"))]
+use jl_sys::jl_opaque_closure_typename;
+#[cfg(feature = "lts")]
+use jl_sys::jl_vararg_typename;
 use std::{marker::PhantomData, ptr::NonNull};
 
+#[cfg(not(feature = "lts"))]
+use super::atomic_value;
+#[cfg(not(feature = "lts"))]
+use std::sync::atomic::Ordering;
+
 /// Describes the syntactic structure of a type and stores all data common to different
 /// instantiations of the type.
 #[derive(Copy, Clone, PartialEq)]
@@ -29,12 +40,15 @@ impl<'scope> TypeName<'scope> {
     name: Symbol
     module: Module
     names: Core.SimpleVector
+    atomicfields: Ptr{Nothing}
     wrapper: Type
     cache: Core.SimpleVector
     linearcache: Core.SimpleVector
-    hash: Int64
     mt: Core.MethodTable
     partial: Any
+    hash: Int64
+    n_uninitialized: Int32
+    flags: UInt8
     */
 
     /// The `name` field.
@@ -48,10 +62,16 @@ impl<'scope> TypeName<'scope> {
     }
 
     /// Field names.
-    pub fn names(self) -> SimpleVectorRef<'scope> {
+    pub fn names(self) -> SimpleVectorRef<'scope, Symbol<'scope>> {
         unsafe { SimpleVectorRef::wrap(self.unwrap_non_null(Private).as_ref().names) }
     }
 
+    /// The `atomicfields` field.
+    #[cfg(not(feature = "lts"))]
+    pub fn atomicfields(self) -> *const u32 {
+        unsafe { self.unwrap_non_null(Private).as_ref().atomicfields }
+    }
+
     /// Either the only instantiation of the type (if no parameters) or a `UnionAll` accepting
     /// parameters to make an instantiation.
     pub fn wrapper(self) -> ValueRef<'scope, 'static> {
@@ -59,18 +79,35 @@ impl<'scope> TypeName<'scope> {
     }
 
     /// Sorted array.
+    #[cfg(feature = "lts")]
     pub fn cache(self) -> SimpleVectorRef<'scope> {
         unsafe { SimpleVectorRef::wrap(self.unwrap_non_null(Private).as_ref().cache) }
     }
 
+    /// Sorted array.
+    #[cfg(not(feature = "lts"))]
+    pub fn cache(self) -> SimpleVectorRef<'scope> {
+        unsafe {
+            let cache = atomic_value(self.unwrap_non_null(Private).as_ref().cache);
+            let ptr = cache.load(Ordering::Relaxed);
+            SimpleVectorRef::wrap(ptr.cast())
+        }
+    }
+
     /// Unsorted array.
+    #[cfg(feature = "lts")]
     pub fn linear_cache(self) -> SimpleVectorRef<'scope> {
         unsafe { SimpleVectorRef::wrap(self.unwrap_non_null(Private).as_ref().linearcache) }
     }
 
-    /// The `hash` field.
-    pub fn hash(self) -> isize {
-        unsafe { self.unwrap_non_null(Private).as_ref().hash }
+    /// Unsorted array.
+    #[cfg(not(feature = "lts"))]
+    pub fn linear_cache(self) -> SimpleVectorRef<'scope> {
+        unsafe {
+            let linearcache = atomic_value(self.unwrap_non_null(Private).as_ref().linearcache);
+            let ptr = linearcache.load(Ordering::Relaxed);
+            SimpleVectorRef::wrap(ptr.cast())
+        }
     }
 
     /// The `mt` field.
@@ -82,6 +119,35 @@ impl<'scope> TypeName<'scope> {
     pub fn partial(self) -> ValueRef<'scope, 'static> {
         unsafe { ValueRef::wrap(self.unwrap_non_null(Private).as_ref().partial.cast()) }
     }
+
+    /// The `hash` field.
+    pub fn hash(self) -> isize {
+        unsafe { self.unwrap_non_null(Private).as_ref().hash }
+    }
+
+    /// The `n_uninitialized` field.
+    #[cfg(not(feature = "lts"))]
+    pub fn n_uninitialized(self) -> i32 {
+        unsafe { self.unwrap_non_null(Private).as_ref().n_uninitialized }
+    }
+
+    /// The `abstract` field.
+    #[cfg(not(feature = "lts"))]
+    pub fn abstract_(self) -> bool {
+        unsafe { self.unwrap_non_null(Private).as_ref().abstract_() != 0 }
+    }
+
+    /// The `mutabl` field.
+    #[cfg(not(feature = "lts"))]
+    pub fn mutabl(self) -> bool {
+        unsafe { self.unwrap_non_null(Private).as_ref().mutabl() != 0 }
+    }
+
+    /// The `mayinlinealloc` field.
+    #[cfg(not(feature = "lts"))]
+    pub fn mayinlinealloc(self) -> bool {
+        unsafe { self.unwrap_non_null(Private).as_ref().mayinlinealloc() != 0 }
+    }
 }
 
 impl<'base> TypeName<'base> {
@@ -100,11 +166,23 @@ impl<'base> TypeName<'base> {
         unsafe { Self::wrap(jl_vecelement_typename, Private) }
     }
 
+    /// The typename of the `UnionAll` `Vararg`.
+    #[cfg(feature = "lts")]
+    pub fn of_vararg(_: Global<'base>) -> Self {
+        unsafe { Self::wrap(jl_vararg_typename, Private) }
+    }
+
     /// The typename of the `UnionAll` `Array`.
     pub fn of_array(_: Global<'base>) -> Self {
         unsafe { Self::wrap(jl_array_typename, Private) }
     }
 
+    /// The typename of the `UnionAll` `Ptr`.
+    #[cfg(not(feature = "lts"))]
+    pub fn of_opaque_closure(_: Global<'base>) -> Self {
+        unsafe { Self::wrap(jl_opaque_closure_typename, Private) }
+    }
+
     /// The typename of the `UnionAll` `Ptr`.
     pub fn of_pointer(_: Global<'base>) -> Self {
         unsafe { Self::wrap(jl_pointer_typename, Private) }
@@ -129,10 +207,12 @@ impl<'scope> Wrapper<'scope, '_> for TypeName<'scope> {
     type Wraps = jl_typename_t;
     const NAME: &'static str = "TypeName";
 
+    #[inline(always)]
     unsafe fn wrap_non_null(inner: NonNull<Self::Wraps>, _: Private) -> Self {
         Self(inner, PhantomData)
     }
 
+    #[inline(always)]
     fn unwrap_non_null(self, _: Private) -> NonNull<Self::Wraps> {
         self.0
     }
